library air_quality;

/*
 * Copyright 2019 Copenhagen Center for Health Technology (CACHET) at the
 * Technical University of Denmark (DTU).
 * Use of this source code is governed by a MIT-style license that can be
 * found in the LICENSE file.
 */
import 'dart:async';
import 'dart:convert';
import 'package:location/location.dart';
import 'package:http/http.dart' as http;

/// Custom Exception for the plugin,
/// Thrown whenever the API responds with an error and body could not be parsed.
class AirQualityAPIException implements Exception {
  String _cause;

  AirQualityAPIException(this._cause);

  String toString() => '${this.runtimeType} - $_cause';
}

enum AirQualityLevel {
  GOOD,
  MODERATE,
  UNHEALTHY_FOR_SENSITIVE_GROUPS,
  UNHEALTHY,
  VERY_UNHEALTHY,
  HAZARDOUS
}

AirQualityLevel airQualityIndexToLevel(int index) {
  if (index < 0)
    throw AirQualityAPIException('Index value cannot be negative!');
  else if (index <= 50)
    return AirQualityLevel.GOOD;
  else if (index <= 100)
    return AirQualityLevel.MODERATE;
  else if (index <= 150)
    return AirQualityLevel.UNHEALTHY_FOR_SENSITIVE_GROUPS;
  else if (index <= 200)
    return AirQualityLevel.UNHEALTHY;
  else if (index <= 300)
    return AirQualityLevel.VERY_UNHEALTHY;
  else
    return AirQualityLevel.HAZARDOUS;
}

/// A class for storing a weather-query response from OpenWeatherMap.
/// This includes various measures such as location,
/// temperature, wind, snow, rain and humidity.
class AirQualityData {
  String _airQualityIndex, _source, _place, _lat, _lon;
  AirQualityLevel _airQualityLevel;

  AirQualityData(Map<String, dynamic> airQualityJson) {
    _airQualityIndex = airQualityJson['data']['aqi'].toString();
    _place = airQualityJson['data']['city']['name'].toString();
    _source = airQualityJson['data']['attributions'][0]['name'].toString();
    _lat = airQualityJson['data']['city']['geo'][0].toString();
    _lon = airQualityJson['data']['city']['geo'][1].toString();
    _airQualityLevel = airQualityIndexToLevel(int.parse(_airQualityIndex));
  }

  String toString() {
    return '''
    Air Quality Level: ${_airQualityLevel.toString().split('.').last}
    AQI: $_airQualityIndex
    Place Name: $_place
    Source: $_source
    Location: ($_lat, $_lon)
    ''';
  }
}

/// Plugin for fetching weather data in JSON.
class AirQuality {
  String _apiKey;
  String _endpoint = 'https://api.waqi.info';

  AirQuality(this._apiKey);

  /// Fetch current weather based on geographical coordinates
  /// Result is JSON.
  /// For API documentation, see: https://openweathermap.org/current
  Future<AirQualityData> _airQualityFromUrl(String url) async {
    try {
      Map<String, dynamic> airQualityJson =
          await _requestAirQualityFromURL(url);
      return AirQualityData(airQualityJson);
    } catch (exception) {
      print(exception);
    }
    return null;
  }

  /// Returns a [AirQualityData] object given a city name.
  Future<AirQualityData> getAirQualityFromCity(String cityName) async {
    String url =
        'https://api.waqi.info/feed/' + '$cityName' + '/?token=' + '$_apiKey';
    return await _airQualityFromUrl(url);
  }

  /// Returns a [AirQualityData] object given a [Location] object.
  /// Checks if location permission is given and requests it if not.
  Future<AirQualityData> getAirQualityFromLocation(Location location) async {
    bool hasPermission = await location.hasPermission();
    print('checking permission');
    if (!hasPermission) {
      await location.requestPermission();
    }
    LocationData gps = await location.getLocation();
    String url =
        '$_endpoint/feed/geo:${gps.latitude};${gps.longitude}/?token=$_apiKey';
    return await _airQualityFromUrl(url);
  }

  /// Send API request given a URL
  Future<Map<String, dynamic>> _requestAirQualityFromURL(String url) async {
    /// Send HTTP get response with the url
    http.Response response = await http.get(url);

    /// Perform error checking on response:
    /// Status code 200 means everything went well
    if (response.statusCode == 200) {
      Map<String, dynamic> jsonBody = json.decode(response.body);
      return jsonBody;
    }
    throw AirQualityAPIException("OpenWeather API Exception: ${response.body}");
  }
}
